unit Main;

interface

uses Windows, Messages;

const
  AppName = 'NkHotKey';
  WM_NKHOTKEY_NOTIFYICON_CALLBACK = WM_APP;
  WM_NKHOTKEY_COMMAND = WM_APP + 1;
  IDM_MENUITEM_START = 20000;

var
  MainWnd: HWND;
  ProfDir: string;
  KeymapFilePath: string;

function MakeWindow: HWND;
procedure MsgBox(const S: string; const Title: string=AppName);
procedure ClearHotKeys;
procedure SuspendHotKeys;
procedure UnsuspendHotKeys;
procedure UpdateHotKeys(Reload: Boolean=True);

implementation

uses NTypes, NLib, NStr, NKeymap, NConf, Default, Command, Version, NotifyIcon;

const
  IDM_SUSPEND = 10000;
  IDM_KEYMAP = 10001;
  IDM_RELOAD = 10002;
  IDM_ABOUT = 10003;
  IDM_EXIT = 10004;

var
  CurrentId: Integer;
  HotKeys: array of TStringArray;
  Keymap: TConf;
  TaskbarCreatedMsg: UINT;

function GetKeyNames(const Command: string): string;
var
  key: string;
begin
  Result := '';
  while Keymap.EachKeys(key) do
    if Keymap[key][0] = Command then
      if Result = '' then
        Result := StrToKeyName(key)
      else
        Result := Result + ', ' + StrToKeyName(key);
end;

procedure About;
var
  msg: string;
  p: TMsgBoxParams;
begin
  msg :=
      AppName+' '+Version_+' ('+ReleaseDate+')'#10+
      #10+
      Copyright+#10+
      'This software is provided under the zlib/libpng license.';
  FillChar(p, SizeOf(p), 0);
  p.cbSize := SizeOf(p);
  p.hwndOwner := MainWnd;
  p.hInstance := MainInstance;
  p.lpszText := PChar(msg);
  p.lpszCaption := PChar('About ' + AppName);
  p.dwStyle := MB_USERICON;
  p.lpszIcon := PChar(1);
  MessageBoxIndirect(p);
end;

procedure ShowKeymap;
var
  i: Integer;
  key: string;
  keys: array of string;
begin
  SetLength(keys, Keymap.Count);
  i := 0;
  while Keymap.EachKeys(key) do
  begin
    keys[i] := key;
    Inc(i);
  end;
  Sort(keys);
  for i := 0 to High(keys) do
    keys[i] := keys[i] + #9 + JoinCommandLine(Keymap[keys[i]]);
  MessageBox(MainWnd,
      PChar('Keymap File: '+KeymapFilePath+#10#10+Join(#10, keys)),
      'Keymap - ' + AppName, MB_OK);
end;

procedure Reload;
begin
  UpdateHotKeys;
  SetNotifyIconInfo('Reload Keymap');
end;

procedure PopupMenu;
var
  menu: HMENU;
  pt: TPoint;
begin
  menu := CreatePopupMenu;
  AppendMenu(menu, MF_STRING, IDM_SUSPEND,
      PChar('&Suspend'#9+GetKeyNames('suspend')));
  if Suspended then
    CheckMenuItem(menu, IDM_SUSPEND, MF_CHECKED);
  AppendMenu(menu, MF_SEPARATOR, 0, nil);
  AppendMenu(menu, MF_STRING, IDM_KEYMAP, 'Show &Keymap');
  AppendMenu(menu, MF_STRING, IDM_RELOAD, '&Reload Keymap');
  AppendMenu(menu, MF_SEPARATOR, 0, nil);
  AppendMenu(menu, MF_STRING, IDM_ABOUT, '&About ' + AppName);
  AppendMenu(menu, MF_SEPARATOR, 0, nil);
  AppendMenu(menu, MF_STRING, IDM_EXIT, '&Exit ' + AppName);
  SetForegroundWindow(MainWnd);
  GetCursorPos(pt);
  TrackPopupMenu(menu, TPM_LEFTALIGN or TPM_LEFTBUTTON or TPM_RIGHTBUTTON,
      pt.x, pt.y, 0, MainWnd, nil);
  DestroyMenu(menu);
end;

function WindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM):
  LRESULT; stdcall;
var
  id: Word;
begin
  Result := 0;
  case Msg of
    WM_NKHOTKEY_NOTIFYICON_CALLBACK:
      if (lParam = WM_LBUTTONUP) or (lParam = WM_RBUTTONUP) then
        PopupMenu;
    WM_NKHOTKEY_COMMAND:
      begin
        CurrentId := wParam;
        DispatchCommand(HotKeys[CurrentId]);
      end;
    WM_HOTKEY:
      if (Length(HotKeys[wParam]) > 0) and (HotKeys[wParam][0] = 'menu') then
      begin
        EndMenu;
        PostMessage(hWnd, WM_NKHOTKEY_COMMAND, wParam, 0);
      end else
        SendMessage(hWnd, WM_NKHOTKEY_COMMAND, wParam, 0);
    WM_COMMAND:
      begin
        id := wParam and $ffff;
        case id of
          IDM_SUSPEND: DispatchCommand(['suspend']);
          IDM_KEYMAP: ShowKeymap;
          IDM_RELOAD: Reload;
          IDM_ABOUT: About;
          IDM_EXIT: PostQuitMessage(0);
        else
          if id >= IDM_MENUITEM_START then
            DispatchCommand(SliceArray(MenuItems[id-IDM_MENUITEM_START], 1));
        end;
      end;
  else
    if Msg = TaskbarCreatedMsg then
      ReAddNotifyIcon
    else
      Result := DefWindowProc(hWnd, Msg, wParam, lParam);
  end;
end;

function MakeWindow: HWND;
const
  classname = AppName + 'WindowClass';
var
  wc: TWndClass;
begin
  TaskbarCreatedMsg := RegisterWindowMessage('TaskbarCreated');
  FillChar(wc, SizeOf(wc), 0);
  wc.lpfnWndProc := @WindowProc;
  wc.hInstance := HInstance;
  wc.lpszClassName := classname;
  if RegisterClass(wc) = 0 then 
    raise EOSError.Create;
  Result := CreateWindowEx(WS_EX_TOOLWINDOW , classname, nil, WS_POPUP, 0, 0,
    64, 64, 0, 0, HInstance, nil);
  if Result = 0 then
    raise EOSError.Create;
end;

procedure MsgBox(const S: string; const Title: string=AppName);
begin
  MessageBox(0, PChar(S), PChar(Title), MB_SETFOREGROUND);
end;

procedure ClearHotKeys;
var
  i: Integer;
begin
  for i := 0 to High(HotKeys) do
    UnregisterHotKey(MainWnd, i);
  Keymap.Clear;
end;

procedure SuspendHotKeys;
var
  i: Integer;
begin
  for i := 0 to High(HotKeys) do
    if i <> CurrentId then
      UnregisterHotKey(MainWnd, i);
end;

procedure UnsuspendHotKeys;
begin
  UnregisterHotKey(MainWnd, CurrentId);
  UpdateHotKeys(False);
end;

procedure UpdateHotKeys(Reload: Boolean=True);
var
  i: Integer;
  key, error: string;
  ks: TKeyState;
  m: UINT;
  vkey: Word;
begin
  if Reload then
  begin
    ClearHotKeys;
    Keymap.Read(Default.Keymap);
    Keymap.LoadFromFile(KeymapFilePath);
    SetLength(HotKeys, Keymap.Count);
  end;

  error := '';
  i := 0;
  while Keymap.EachKeys(key) do
  begin
    StrToVkey(key, ks, vkey);
    m := 0;
    if ksAlt in ks then
      m := m or MOD_ALT;
    if ksCtrl in ks then
      m := m or MOD_CONTROL;
    if ksShift in ks then
      m := m or MOD_SHIFT;
    if ksWin in ks then
      m := m or MOD_WIN;
    try
      if not RegisterHotKey(MainWnd, i, m, vkey) then
        raise EOSError.Create;
      HotKeys[i] := Keymap[key];
      Inc(i);
    except
      on E: EOSError do
        error := error + '  ' + key + ': ' + E.Msg + #10;
    end;
  end;
  if error <> '' then
    MsgBox('register hotkey failed:'#10 + error);
end;

initialization
  Keymap := TConf.Create;

finalization
  Keymap.Free;

end.
