unit NotifyIcon;

interface

procedure AddNotifyIcon;
procedure DeleteNotifyIcon;
procedure ReAddNotifyIcon;
procedure SetNotifyIconState(Suspended: Boolean);
procedure SetNotifyIconInfo(const Info: string);

implementation

uses Windows, ShellAPI, NLib, Main;

const
  NIF_INFO = $10;

type
  // Shell32.dll Version >= 5.00 (Windows 2000 and later)
  TNotifyIconData = record
    cbSize: DWORD;
    Wnd: HWND;
    uID: UINT;
    uFlags: UINT;
    uCallbackMessage: UINT;
    hIcon: HICON;
    szTip: array [0..127] of AnsiChar;
    dwState: DWORD;
    dwStateMask: DWORD;
    szInfo: array [0..255] of AnsiChar;
    uTimeout: UINT;
    szInfoTitle: array [0..63] of AnsiChar;
    dwInfoFlags: DWORD;
  end;

var
  NotifyIconData: TNotifyIconData;
  AppIcon: HICON;
  SuspendedIcon: HICON;

procedure AddNotifyIcon;
begin
  AppIcon := LoadImage(HInstance, PChar(1), IMAGE_ICON, 16, 16,
      LR_DEFAULTCOLOR);
  SuspendedIcon := LoadImage(HInstance, PChar(2), IMAGE_ICON, 16, 16,
      LR_DEFAULTCOLOR);
  NotifyIconData.cbSize := SizeOf(TNotifyIconData);
  NotifyIconData.Wnd := MainWnd;
  NotifyIconData.uID := 1;
  NotifyIconData.uFlags := NIF_ICON or NIF_MESSAGE or NIF_TIP or NIF_INFO;
  NotifyIconData.uCallbackMessage := WM_NKHOTKEY_NOTIFYICON_CALLBACK;
  NotifyIconData.hIcon := AppIcon;
  NotifyIconData.szTip := AppName;
  NotifyIconData.szInfoTitle := AppName;
  if not Shell_NotifyIcon(NIM_ADD, @NotifyIconData) then
    raise EOSError.Create;
end;

procedure DeleteNotifyIcon;
begin
  if not Shell_NotifyIcon(NIM_DELETE, @NotifyIconData) then
    raise EOSError.Create;
end;

procedure ReAddNotifyIcon;
begin
  if not Shell_NotifyIcon(NIM_ADD, @NotifyIconData) then
    raise EOSError.Create;
end;

procedure SetNotifyIconState(Suspended: Boolean);
begin
  if Suspended then
  begin
    NotifyIconData.hIcon := SuspendedIcon;
    NotifyIconData.szTip := AppName + #13#10'Suspended';
    NotifyIconData.szInfo := 'Suspended';
  end
  else begin
    NotifyIconData.hIcon := AppIcon;
    NotifyIconData.szTip := AppName;
    NotifyIconData.szInfo := 'Unsuspended';
  end;
  if not Shell_NotifyIcon(NIM_MODIFY, @NotifyIconData) then
    raise EOSError.Create;
end;

procedure SetNotifyIconInfo(const Info: string);
var
  len: Integer;
begin
  len := Length(Info);
  if len > (SizeOf(NotifyIconData.szInfo)-1) then
    len := SizeOf(NotifyIconData.szInfo)-1;
  Move(Info[1], NotifyIconData.szInfo, len+1); // +1 for null terminate
  if not Shell_NotifyIcon(NIM_MODIFY, @NotifyIconData) then
    raise EOSError.Create;
end;

end.
