unit WinMan;

interface

uses Windows;

const
  spUndef = $7fffffff;

type
  TPositionOrigin = (poLeftTop, poLeftBottom, poRightTop, poRightBottom);

procedure Foreground(const ClassName: string; Prev: Boolean);
procedure GetInfo(var WindowName: string; var WindowHandle: HWND;
    var ClassName: string; var X, Y, W, H: Longint);
procedure Minimize;
procedure MinimizeAll;
procedure SetPosition(X, Y, W, H: Longint; Origin: TPositionOrigin);

implementation

uses Messages, Main;

function GetClassName(Wnd: HWND): string;
begin
  SetString(Result, nil, 256);
  SetLength(Result, Windows.GetClassName(wnd, PChar(Result), Length(Result)));
end;

procedure Foreground(const ClassName: string; Prev: Boolean);
type
  TData = record
    ClassName: string;
    ForegroundWnd: HWND;
    FoundFWnd: Boolean;
    FirstWnd: HWND;
    LastWnd: HWND;
    NextWnd: HWND;
    PrevWnd: HWND;
  end;
  PData = ^TData;

  function enumwindowsproc(Wnd: HWND; lParam: LPARAM): BOOL; stdcall;
  begin
    Result := True;
    if not IsWindowVisible(Wnd) then
      Exit;
    with PData(lParam)^ do
    begin
      if GetClassName(Wnd) = ClassName then
      begin
        if FirstWnd = 0 then
          FirstWnd := Wnd;
        LastWnd := Wnd;
      end;

      if Wnd = ForegroundWnd then
      begin
        FoundFWnd := True;
        Exit;
      end;

      if GetClassName(Wnd) = ClassName then
        if FoundFWnd then
        begin
          if NextWnd = 0 then
            NextWnd := Wnd
        end else
          PrevWnd := Wnd;
    end;
  end;
var
  wnd: HWND;
  data: TData;
begin
  data.ClassName := ClassName;
  data.ForegroundWnd := GetForegroundWindow;
  data.FoundFWnd := False;
  data.FirstWnd := 0;
  data.LastWnd := 0;
  data.NextWnd := 0;
  data.PrevWnd := 0;
  EnumWindows(@enumwindowsproc, LPARAM(@data));
  if data.PrevWnd = 0 then
    data.PrevWnd := data.LastWnd;
  if data.NextWnd = 0 then
    data.NextWnd := data.FirstWnd;
  if Prev then
  begin
    if data.PrevWnd = 0 then
      Exit;
    wnd := data.PrevWnd;
  end
  else begin
    if data.NextWnd = 0 then
      Exit;
    wnd := data.NextWnd;
  end;
  if IsIconic(wnd) then
    PostMessage(wnd, WM_SYSCOMMAND, SC_RESTORE, 0);
  SetForegroundWindow(wnd);
  if (not Prev) and (GetClassName(data.Foregroundwnd) = ClassName) then
    SetWindowPos(data.ForegroundWnd, data.LastWnd, 0, 0, 0, 0,
        SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE or SWP_NOOWNERZORDER);
end;

procedure GetInfo(var WindowName: string; var WindowHandle: HWND;
    var ClassName: string; var X, Y, W, H: Longint);
var
  wp: TWindowPlacement;
begin
  WindowHandle := GetForegroundWindow;
  ClassName := GetClassName(WindowHandle);
  SetString(WindowName, nil, 255);
  SetLength(WindowName,
      GetWindowText(WindowHandle, PChar(WindowName), Length(WindowName)));
  wp.length := SizeOf(wp);
  if GetWindowPlacement(WindowHandle, @wp) then
    with wp.rcNormalPosition do
    begin
      X := Left;
      Y := Top;
      W := Right - Left;
      H := Bottom - Top;
    end
  else begin
    X := 0;
    Y := 0;
    W := 0;
    H := 0;
  end;
end;

procedure _Minimize(Wnd: HWND);
var
  rc: TRect;
  ws: Longint;
begin
  ws := GetWindowLong(Wnd, GWL_STYLE);
  if ((ws and WS_VISIBLE) = 0) or
      ((ws and WS_MINIMIZE) <> 0) or
      ((ws and WS_MINIMIZEBOX) = 0) or
      ((GetWindowLong(Wnd, GWL_EXSTYLE) and WS_EX_TOOLWINDOW) <> 0) or
      not GetWindowRect(Wnd, rc) or
      ((rc.Right-rc.Left) = 0) or ((rc.Bottom-rc.Top) = 0) then
    Exit;
  PostMessage(Wnd, WM_SYSCOMMAND, SC_MINIMIZE, 0);
end;

procedure Minimize;
begin
  _Minimize(GetForegroundWindow);
end;

procedure MinimizeAll;
  function enumwindowsproc(Wnd: HWND; lParam: LPARAM): BOOL; stdcall;
  begin
    _Minimize(Wnd);
    Result := True;
  end;
begin
  EnumWindows(@enumwindowsproc, 0);
end;

procedure SetPosition(X, Y, W, H: Longint; Origin: TPositionOrigin);
  function getval(Value, Default: Longint): Longint;
  begin
    if Value = spUndef then
      Result := Default
    else
      Result := Value;
  end;
var
  wnd: HWND;
  src, rc: TRect;
  wp: TWindowPlacement;
begin
  wnd := GetForegroundWindow;
  if (GetWindowLong(wnd, GWL_STYLE) and WS_THICKFRAME) = 0 then
    Exit;
  GetWindowRect(wnd, rc);
  SystemParametersInfo(SPI_GETWORKAREA, 0, @src, 0);
  W := getval(W, rc.Right - rc.Left);
  H := getval(H, rc.Bottom - rc.Top);
  if Origin in [poRightTop, poRightBottom] then
    X := src.Right - W - X;
  if Origin in [poLeftBottom, poRightBottom] then
    Y := src.Bottom - H - Y;
  X := getval(X, rc.Left);
  Y := getval(Y, rc.Top);
  wp.length := SizeOf(wp);
  if GetWindowPlacement(wnd, @wp) then
  begin
    with wp.rcNormalPosition do
    begin
      Left := X;
      Top := Y;
      Right := X + W;
      Bottom := Y + H;
    end;
    SetWindowPlacement(wnd, @wp);
  end;
end;

end.
